% Model of the flow of air around a cylinder, ignoring turbulence effects
% etc.
function cylinder

%Cylinder centre coordinates /m
xc = 0; yc = 0;

%Define radius of cylinder /m
R = 1.5;

%Angle of attack of air flow /radians
a = 0*pi/180;

%Airspeed away from cylinder /ms^-1
U = 0;

%Define (clockwise) rotation frequency of cylinder /Hz
frot = 2;

%Argand diagram limits
xlimits = [-5,5]; ylimits = [-4,4];

%Fontsize for graphs
fsize = 16;

%

%Define circle of radius R, centre xy,yc
theta = linspace(0,2*pi,300);

%Convert to Cartesians
xx = R*cos(theta)+ xc; yy = R*sin(theta) + yc;

%Determine velocity field around cylinder
[xq,yq,vxq,vyq] = airfoil( xc,yc, R,a, U, frot,50,xlimits,ylimits);
[x,y,vx,vy] = airfoil( xc,yc, R, a, U, frot, 1000,xlimits,ylimits);

%Plot velocities
pcolor(x,y,sqrt(vx.^2 + vy.^2 ) ); shading interp; hold on; axis tight; box on;
plot( xx,yy,'r-','linewidth',2);
set(gca,'fontsize',fsize); set( gcf,'units','normalized','position',[0 0 1 1] );
caxis([0,2*U]); colorbar('fontsize',fsize); colormap('jet'); icmap;
quiver( xq,yq,vxq,vyq );
axis equal; xlim(xlimits); ylim(ylimits);
title(['U=',num2str(U),',   \alpha=',num2str(a*180/pi),'^o,  f_{rot}=',num2str(frot),'Hz']);

%Print PNG output
print( gcf,'cylinder.png','-dpng','-r300');

%%

%Cylinder flow function
function [x,y,vx,vy] = airfoil( xc,yc, R, a, U, frot, N,xlimits,ylimits)

% Define circulation F (e.g. rotating cylinder)
% Assume tangential speed = 2*pi*frot*R = F/(2*pi*R)
F = 4*frot*(R*pi)^2;

%Define Argand diagram
x = linspace( xlimits(1), xlimits(2), N );
y = linspace( ylimits(1), ylimits(2), N); 
[x,y] = meshgrid(x,y); z = x + 1i*y;

%Shift such that centre of circle is the origin
z = z - xc - 1i*yc;

%Set as NaN the velocities within a circle of radius R about the origin
z( abs(z) <=R ) = NaN; 

%Define complex velocity
W = U*exp(-1i*a) + 1i*( F/(2*pi) )./z -...
    U*(R^2)*exp(1i*a)./( z.^2 );

%Define velocities
vx = real(W); vy = -imag(W);

%%

%Interpolates current colormap to yield better graduated shading.
function icmap
N = 1000; %Length of new colormap
map = colormap; new_map = ones(N,3); %Get current colormap

%Get size of current colormap and initalise R,G,B vectors
dim = size(map);
R = ones(1,dim(1)); G = ones(1,dim(1)); B = ones(1,dim(1));
RR = ones(1,N); GG = ones(1,N); BB = ones(1,N);

%Populate R,G,B with current colormap
R(:) = map(:,1); G(:) = map(:,2); B(:) = map(:,3);

%Interpolate R,G,B to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x ); GG = interp1( 1:dim(1), G, x ); BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:); new_map(:,2) = GG(:); new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code

