% Model of the flow of air around an airfoil, ignoring turbulence effects
% etc. This is achieved via the joukowski complex number transformation.
% https://en.wikipedia.org/wiki/Joukowsky_transform
function joukowski

%Airfoil 'centre' coordinates
xc = -0.08; yc = 0.08;

%Angle of attack of aerofoil /radians
a = 10*pi/180;

%Airspeed away from aerofoil /ms^-1
U = 10;

%Argand diagram limits
xlimits = [-5,5]; ylimits = [-4,4];

%Fontsize for graphs
fsize = 16;

%

%Define circle of radius R, centre xy,yc
theta = linspace(0,2*pi,300);
R = sqrt( (1-xc)^2 + yc^2 );
xx = R*cos(theta)+ xc; yy = R*sin(theta) + yc;

%Apply Joukowski transformation to circle to define airfoil shape
[xx,yy] = jtrans(xx,yy); [xx,yy] = rot( a,xx,yy,0,0 );

%Determine velocity field around airfoil
[xq,yq,vxq,vyq] = airfoil( xc,yc, a, U, 50,xlimits,ylimits);
[x,y,vx,vy] = airfoil( xc,yc, a, U, 1000,xlimits,ylimits);

%Plot velocities
pcolor(x,y,sqrt(vx.^2 + vy.^2 ) ); shading interp; hold on; axis tight; box on;
plot( xx,yy,'r-','linewidth',2);
set(gca,'fontsize',fsize); set( gcf,'units','normalized','position',[0 0 1 1] );
caxis([0,2*U]); colorbar('fontsize',fsize); colormap('jet'); icmap;
quiver( xq,yq,vxq,vyq );
axis equal; xlim(xlimits); ylim(ylimits);
title(['Joukowski airfoil. Colour is airspeed (m/s). U=',num2str(U),...
    ',   \alpha=',num2str(a*180/pi),'^o']);

%Print PNG output
print( gcf,'joukowski.png','-dpng','-r300');

%%

%Joukowsky airfoil function
function [x,y,vx,vy] = airfoil( xc,yc, a, U, N,xlimits,ylimits)

%Define circle radius R and ciculation F, which satisfies the Kutta criteria.
%This means a stagnation point for (A,0) on the circle, which ends up at
%the trailing edge fo the Joukowski airfoil at (0,2*A) in unrotated
%coordinates (i.e a = 0).
R = sqrt( (1-xc)^2 + yc^2 );
F = 4*pi*U*R*sin( a + asin( yc/R ) );
zc = xc + 1i*yc;

%Define Argand diagram
x = linspace( xlimits(1), xlimits(2), N );
y = linspace( ylimits(1), ylimits(2), N); 
[x,y] = meshgrid(x,y); z = x + 1i*y;

%Set as NaN the velocities within the circle
z = z - xc -1i*yc; z( abs(z) <=R ) = NaN; z = z + xc + 1i*yc;

%Define complex velocity
W = U*exp(-1i*a) + 1i*( F/(2*pi) )./(z - zc) -...
    U*(R^2)*exp(1i*a)./( (z - zc).^2 );

%Apply Joukowski conformal mapping
W = W./( 1 - z.^(-2) ); [x,y] = jtrans(x,y);

%Define velocities
vx = real(W); vy = -imag(W);

%Rotate entire space and velocities by a clockwise (i.e. so far-field
%velocity is horizontal left to right)
[x,y] = rot( a,x,y,0,0 ); [vx,vy] = rot( a,vx,vy,x,y );

%%

%Joukowski conformal mapping
function [x,y] = jtrans(x,y)
z = x + 1i*y; z = z + 1./z;
x = real(z); y  = imag(z);

%%

%Rotate clockwise by theta (radians) about points (xr,yr)
function [xx,yy] = rot( theta, x,y,xr,yr )
x = x - xr; y = y - yr;
dim = size(x); x = reshape( x, [1,numel(x)] ); y = reshape( y, [1,numel(y)] );
xy = [ cos(theta), sin(theta); -sin(theta), cos(theta) ] * [x;y];
xx = xy(1,:); yy = xy(2,:);
xx = reshape( xx,dim ) + xr; yy = reshape( yy,dim ) + yr;

%%

%Interpolates current colormap to yield better graduated shading.
function icmap
N = 1000; %Length of new colormap
map = colormap; new_map = ones(N,3); %Get current colormap

%Get size of current colormap and initalise R,G,B vectors
dim = size(map);
R = ones(1,dim(1)); G = ones(1,dim(1)); B = ones(1,dim(1));
RR = ones(1,N); GG = ones(1,N); BB = ones(1,N);

%Populate R,G,B with current colormap
R(:) = map(:,1); G(:) = map(:,2); B(:) = map(:,3);

%Interpolate R,G,B to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x ); GG = interp1( 1:dim(1), G, x ); BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:); new_map(:,2) = GG(:); new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code

